@@ -65,10 +65,12 @@ static const commands_st commands[] = {
 	      "Prints all the known IP addresses which have points", 1, 1),
 	ENTRY("show iroutes", NULL, handle_list_iroutes_cmd,
 	      "Prints the routes provided by users of the server", 1, 1),
-	ENTRY("show sessions all", NULL, handle_list_all_cookies_cmd,
+	ENTRY("show sessions all", NULL, handle_list_all_sessions_cmd,
 	      "Prints all the session IDs", 1, 1),
-	ENTRY("show sessions valid", NULL, handle_list_valid_cookies_cmd,
+	ENTRY("show sessions valid", NULL, handle_list_valid_sessions_cmd,
 	      "Prints all the valid for reconnection sessions", 1, 1),
+	ENTRY("show session", "[SID]", handle_show_session_cmd,
+	      "Prints information on the specified session", 1, 1),
 	ENTRY("show user", "[NAME]", handle_show_user_cmd,
 	      "Prints information on the specified user", 1, 1),
 	ENTRY("show id", "[ID]", handle_show_id_cmd,
@@ -84,9 +86,9 @@ static const commands_st commands[] = {
 	/* hidden options */
 	ENTRY("?", NULL, handle_help_cmd, "Prints this help", -1, 0),
 	ENTRY("quit", NULL, handle_exit_cmd, "Exits this application", -1, 0),
-	ENTRY("show cookies all", NULL, handle_list_all_cookies_cmd,
+	ENTRY("show cookies all", NULL, handle_list_all_sessions_cmd,
 	      "Alias for show sessions all", -1, 1),
-	ENTRY("show cookies valid", NULL, handle_list_valid_cookies_cmd,
+	ENTRY("show cookies valid", NULL, handle_list_valid_sessions_cmd,
 	      "Alias for show sessions valid", -1, 1),
 	{NULL, 0, NULL, NULL}
 };
@@ -458,6 +460,10 @@ static char *command_generator(const char *text, int state)
 						ret =
 						    search_for_ip(entries_idx,
 								  text, len);
+					else if (strcmp(arg, "[SID]") == 0)
+						ret =
+						    search_for_session(entries_idx,
+								  text, len);
 					if (ret != NULL) {
 						entries_idx++;
 					}
